;;; ewb-mode.el --- Setup ewb-mode for EWB.
;;
;; Copyright (C) 1999 ASCII Corporation
;;
;; This file is part of EWB.
;;
;;; Code:

(require 'ewb-tools)
(require 'ewb-index)
(require 'ewb-check)
;(require 'ewb-buf)
(require 'ewb-triger)
(require 'ewb-phy)
(require 'novice-search)

(defvar ewb-mode-init-file-name "~/.xewbindex"
  "Init file for EWB mode")

(defvar ewb-shelf-file-name  ".shelf2"
  "EWB shelf file")

(defvar ewb-font-lock-mode-p t
  "Use EWB Font Lock mode?")

(defvar ewb-mode-syntax-table nil
  "Syntax table used while in EWB mode.")

(defvar ewb-mode-abbrev-table nil
  "Abbrev table used while in EWB mode.")
(define-abbrev-table 'ewb-mode-abbrev-table ())

;(defvar fume-section-name-regexp-ewb
;  "^//i+ [^ ]*"
;  "Expression to get EWB section names")

;(setq fume-function-name-regexp-alist
;      (cons
;       '(
;	 (ewb-mode              . fume-section-name-regexp-ewb)
;	 (EWB-mode              . fume-section-name-regexp-ewb)
;	 )
;       fume-function-name-regexp-alist))

(if ewb-mode-syntax-table
    ()
  (setq ewb-mode-syntax-table (make-syntax-table))
  (modify-syntax-entry ?\" ".   " ewb-mode-syntax-table)
  (modify-syntax-entry ?\\ ".   " ewb-mode-syntax-table)
  (modify-syntax-entry ?' "w   " ewb-mode-syntax-table))

(defvar ewb-mode-map nil
  "Keymap for EWB mode.
Many other modes, such as Mail mode, Outline mode and Indented EWB mode,
inherit all the commands defined in this map.")

(if ewb-mode-map
    ()
  (setq ewb-mode-map (make-sparse-keymap))
  ;; XEmacs change
  (set-keymap-name ewb-mode-map 'ewb-mode-map)
  (define-key ewb-mode-map "\C-ci" 'ewb-index)
  (define-key ewb-mode-map "\C-ck" 'ewb-remove-index)
  (define-key ewb-mode-map "\C-cj" 'ewb-index-jump-to-forall)
  (define-key ewb-mode-map "\C-c\C-f" 'novice-search)
  (define-key ewb-mode-map "\C-c\C-s" 'novice-search-repeat-forward)
  (define-key ewb-mode-map "\C-c\C-r" 'novice-search-repeat-backward)
  (define-key ewb-mode-map "\C-cc" 'ewb-insert-triger)
  (define-key ewb-mode-map "\C-cs" 'ewb-insert-section)
  (define-key ewb-mode-map "\C-cf" 'ewb-insert-figure)
;  (define-key ewb-mode-map "\C-c\C-t" 'ewb-typeset)
;  (define-key ewb-mode-map "\C-c\C-v" 'ewb-view)
;  (define-key ewb-mode-map "\C-c\C-p" 'ewb-print)
;  (define-key ewb-mode-map "\C-c\C-c" 'ewb-clean)
  (define-key ewb-mode-map "\C-c\C-f" 'ewb-view-figure)
  (define-key ewb-mode-map "\C-c\C-e" 'ewb-edit-phy)
  )


(defun ewb-mode ()
  "Major mode for editing EWB intended for humans to read.
Special commands:
\\{ewb-mode-map}
Turning on EWB mode calls the value of the variable `ewb-mode-hook',
if that value is non-nil."
  (interactive)
  (kill-all-local-variables)
  (use-local-map ewb-mode-map)
  (setq mode-name "EWB")
  (setq major-mode 'ewb-mode)
  (setq local-abbrev-table ewb-mode-abbrev-table)
  (set-syntax-table ewb-mode-syntax-table)
  (setq fill-column 100000000)
  (setq tab-width 4)

  ;; outline minor mode
  (setq outline-regexp "^\\(chapter\\)\\|\\(//i+\\)[^a-zA-Z]")
  (setq outline-minor-mode-prefix "\C-c\C-o")
  (outline-minor-mode t)

  (if (and (featurep 'toolbar)
	   (console-on-window-system-p))
      (set-specifier default-toolbar (cons (current-buffer) ewb::toolbar)))
  (if (featurep 'menubar)
      (progn
	;; make a local copy of the menubar, so our modes don't
	;; change the global menubar
	(set-buffer-menubar current-menubar)
	(add-submenu nil ewb-shelf-menu)
	(add-submenu nil ewb-menu)
	;; In an effort to avoid massive menu clutter, this mostly worthless menu is
	;; superceded by any local popup menu...
	(setq mode-popup-menu ewb-mode-popup-menu)
	))
  (if ewb-font-lock-mode-p
      (font-lock-mode t))
  (setq line-number-mode t)

  (ewb-init)

  (run-hooks 'ewb-mode-hook))

(defun ewb-init ()
  (interactive)
  (if (file-exists-p ewb-index-file-name)
      (progn
	(if (one-window-p)
	    (split-window-vertically (- (window-height) 10)))
	(set-window-buffer (next-window) (find-file-noselect ewb-index-file-name)))))


;;; EWB font-lock support

(put 'ewb-mode 'font-lock-defaults
     '(ewb-font-lock-keywords nil nil nil))

(defconst ewb-font-lock-keywords (purecopy
  (list
   '("////" . font-lock-function-name-face)
;   '("\\(//in\\([0-9]+\\)\\)[^0-9]" 1 font-lock-comment-face)
;   '("\\(//in\\)[^a-zA-Z]" 1 font-lock-comment-face)
   '("\\(//in\\([0-9]+\\)\\)[^0-9]" 1 font-lock-type-face)
   '("\\(//in\\)[^a-zA-Z]" 1 font-lock-type-face)
   '("//[a-zA-z0-9]+\\[.*\\]{" . font-lock-function-name-face)
   '("//[a-zA-z0-9]+[{}]?" . font-lock-function-name-face)
   '("//[|}]" . font-lock-function-name-face)
   '("@@[a-zA-Z0-9]+(\\(-?[0-9\.]+\\([a-zA-Z]+\\)?\\)?)" . font-lock-string-face)
   '("@@@@" . font-lock-function-name-face)
   '("^chapter [0-9]+$" . font-lock-function-name-face)
   ))
  "Additional expressions to highlight in ewb-mode.")



;;; EWB toolbar support

(defvar ewb-toolbar-icon-directory
  "/usr/share/ewb/3.3/xemacs/etc")

(defcustom toolbar-ewbindex-function 'ewb-index
  "*Function to call when the copy icon is selected."
  :type '(radio (function-item x-copy-primary-selection)
                (function :tag "Other"))
  :group 'toolbar)

(defun toolbar-ewbindex ()
  (interactive)
  (call-interactively toolbar-ewbindex-function))

(defcustom toolbar-ewbindex-jump-function 'ewb-index-jump-forall
  "*Function to call when the copy icon is selected."
  :type '(radio (function-item x-copy-primary-selection)
                (function :tag "Other"))
  :group 'toolbar)

(defun toolbar-ewbindex-jump ()
  (interactive)
  (call-interactively toolbar-ewbindex-jump-function))

(defcustom toolbar-novice-search-function 'novice-search
  "*Function to call when the copy icon is selected."
  :type '(radio (function-item x-copy-primary-selection)
                (function :tag "Other"))
  :group 'toolbar)

(defun toolbar-novice-search ()
  (interactive)
  (call-interactively toolbar-novice-search-function))

(defun ewb-stop ()
  (interactive)
  (if (> (recursion-depth) 0)
	(throw 'exit t))
  )

(defun toolbar-ewb-info ()
  "Run info in a separate frame."
  (interactive)
  (if (or (not toolbar-info-frame)
	  (not (frame-live-p toolbar-info-frame)))
      (progn
	(setq toolbar-info-frame (make-frame Info-frame-plist))
	(select-frame toolbar-info-frame)
	(raise-frame toolbar-info-frame)))
  (if (frame-iconified-p toolbar-info-frame)
      (deiconify-frame toolbar-info-frame))
  (select-frame toolbar-info-frame)
  (raise-frame toolbar-info-frame)
  (info "ewb-mode"))

(defcustom toolbar-novice-search-repeat-forward-function 'novice-search-repeat-forward
  "*Function to call when the copy icon is selected."
  :type '(radio (function-item x-copy-primary-selection)
                (function :tag "Other"))
  :group 'toolbar)

(defun toolbar-novice-search-repeat-forward ()
  (interactive)
  (call-interactively toolbar-novice-search-repeat-forward-function))

(defcustom toolbar-novice-search-repeat-backward-function 'novice-search-repeat-backward
  "*Function to call when the copy icon is selected."
  :type '(radio (function-item x-copy-primary-selection)
                (function :tag "Other"))
  :group 'toolbar)

(defun toolbar-novice-search-repeat-backward ()
  (interactive)
  (call-interactively toolbar-novice-search-repeat-backward-function))

(defvar ewb::toolbar-stop-icon
  (if (featurep 'toolbar)
      (toolbar-make-button-list
       (expand-file-name (if (featurep 'xpm) "ewb-stop.xpm" "ewb-stop.xbm")
			 ewb-toolbar-icon-directory)))
  "EWB jump icon")

(defvar ewb::toolbar-jump-icon
  (if (featurep 'toolbar)
      (toolbar-make-button-list
       (expand-file-name (if (featurep 'xpm) "ewb-jump.xpm" "ewb-jump.xbm")
			 ewb-toolbar-icon-directory)))
  "EWB jump icon")

(defvar ewb::toolbar-index-icon
  (if (featurep 'toolbar)
      (toolbar-make-button-list
       (expand-file-name (if (featurep 'xpm) "ewb-sakuin.xpm" "ewb-sakuin.xbm")
			  ewb-toolbar-icon-directory)))
  "EWB index icon")

(defvar ewb::toolbar-novice-search-icon
  (if (featurep 'toolbar)
      (toolbar-make-button-list
       (expand-file-name (if (featurep 'xpm) "novice-search.xpm" "novice-search.xbm")
			  ewb-toolbar-icon-directory)))
  "Novice search icon")

(defvar ewb::toolbar-novice-search-repeat-forward-icon
  (if (featurep 'toolbar)
      (toolbar-make-button-list
       (expand-file-name (if (featurep 'xpm) "novice-forward.xpm" "novice-forward.xbm")
			  ewb-toolbar-icon-directory)))
  "Novice search forward icon")

(defvar ewb::toolbar-novice-search-repeat-backward-icon
  (if (featurep 'toolbar)
      (toolbar-make-button-list
       (expand-file-name (if (featurep 'xpm) "novice-backward.xpm" "novice-backward.xbm")
			  ewb-toolbar-icon-directory)))
  "Novice search backward icon")

(defvar ewb::toolbar
  (if (featurep 'toolbar)
; disabled until we get the next/prev-win icons working again.
;      (cons (first initial-toolbar-spec)
;       (cons (second initial-toolbar-spec)
	     '(
	       [toolbar-file-icon
		toolbar-open	t	"Open a file"]
	       [toolbar-disk-icon
		toolbar-save	t	"Save buffer"]
	       [toolbar-cut-icon
		toolbar-cut	t	"Kill region"]
	       [toolbar-copy-icon
		toolbar-copy	t	"Copy region"]
	       [toolbar-paste-icon
		toolbar-paste	t	"Paste from clipboard"]
	       [toolbar-undo-icon
		toolbar-undo	t	"Undo edit"]
	       [toolbar-replace-icon
		toolbar-replace	t	"Replace text"]
	       [ewb::toolbar-novice-search-icon
		toolbar-novice-search	t	"Novice search"]
	       [ewb::toolbar-novice-search-repeat-forward-icon
		toolbar-novice-search-repeat-forward	t	"Novice search forward"]
	       [ewb::toolbar-novice-search-repeat-backward-icon
		toolbar-novice-search-repeat-backward	t	"Novice search backward"]
	       [ewb::toolbar-jump-icon
		toolbar-ewbindex-jump	t	"EWB jump"]
	       [ewb::toolbar-index-icon
		toolbar-ewbindex	t	"EWB index"]
	       [ewb::toolbar-stop-icon
		ewb-stop	t	"Stop"]
;	       nil
;	       [toolbar-info-icon
;		toolbar-ewb-info	t	"EWB Information"]
		)))


;;; EWB menu support

(defvar ewb-shelf-menu
  '("Shelf"
    :filter ewb-shelf-menu-filter
    )
  )

(defvar ewb-menu
  '("EWB"
    :filter ewb-menu-filter
    ["Insert Triger"  	ewb-insert-triger		t]
    ["Insert Section"  	ewb-insert-section		t]
    ["Insert Figure"  	ewb-insert-figure		t]
    ["View Figure"	ewb-view-figure			t]
    ["Edit/Add Figure"	ewb-edit-phy			t]
;    "-"
;    ["Typeset"  	ewb-typeset			t]
;    ["View"		ewb-view			t]
;    ["Print"		ewb-print			t]
;    ["Clean"	  	ewb-clean			t]
    "-"
    ["Make/Edit index"  ewb-index			t]
    ["Remove index"  	ewb-remove-index		t]
    ["Jumpt to entry"  	ewb-index-jump-to-forall	t]
    ["Convert all //idx{..//}"  ewb-convert-all-idx	t]
    ("Check"
     ["No entry"  	ewb-check-not-in-forall		t]
     ["Doubled"  	ewb-check-double-index		t]
     )
    ("Options"
     ("//in position"
      ["Forward"
       (progn
	 (setq ewb-index-position-forward t)
	 (let ((ibuf (find-file-noselect ewb-mode-init-file-name))
	       (obuf (current-buffer)))
	   (progn
	     (set-buffer ibuf)
	     (goto-char (point-min))
	     (delete-matching-lines "^(setq ewb-index-position-forward nil)")
	     (goto-char (point-max))
	     (insert "(setq ewb-index-position-forward t)\n")
	     (save-buffer)
	     (set-buffer obuf))))
       :style radio
       :selected ewb-index-position-forward ]
      ["Backward"
       (progn
	 (setq ewb-index-position-forward nil)
	 (let ((ibuf (find-file-noselect ewb-mode-init-file-name))
	       (obuf (current-buffer)))
	   (progn
	     (set-buffer ibuf)
	     (goto-char (point-min))
	     (delete-matching-lines "^(setq ewb-index-position-forward t)")
	     (goto-char (point-max))
	     (insert "(setq ewb-index-position-forward nil)\n")
	     (save-buffer)
	     (set-buffer obuf))))
       :style radio
       :selected (not ewb-index-position-forward) ]
      )
     ["Color"
      (let ((ibuf (find-file-noselect ewb-mode-init-file-name))
	    (obuf (current-buffer)))
	(progn
	  (set-buffer ibuf)
	  (goto-char (point-min))
	  (if ewb-font-lock-mode-p
	      (delete-matching-lines "^(setq ewb-font-lock-mode-p t)")
	    (delete-matching-lines "^(setq ewb-font-lock-mode-p nil)"))
	  (goto-char (point-max))
	  (if ewb-font-lock-mode-p
	      (insert "(setq ewb-font-lock-mode-p nil)\n")
	    (insert "(setq ewb-font-lock-mode-p t)\n"))
	  (save-buffer)
	  (set-buffer obuf)
	  (princ "εưͭˤʤޤ")))
       :style radio
       :selected ewb-font-lock-mode-p ]
     )
    ))


;;; Popup menus.

(defconst ewb-mode-popup-menu
  '("EWB edit Commands"
    :filter ewb-menu-filter
    ["Insert Triger"  	ewb-insert-triger		t]
    ["Insert Section"  	ewb-insert-section		t]
    ["Insert Figure"  	ewb-insert-figure		t]
    ["View Figure"	ewb-view-figure			t]
    ["Edit/Add Figure"	ewb-edit-phy			t]
    ["Make/Edit index"  	ewb-index		t]
    ["Jumpt to entry"  	ewb-index-jump-to-forall	t]
    ["Remove index"  	ewb-remove-index		t]
    "-"
    ["Undo"		advertised-undo		t]
    ["Cut"		x-kill-primary-selection   t]
    ["Copy"		x-copy-primary-selection   t]
    ["Paste"		x-yank-clipboard-selection t]
    ["Clear"            x-delete-primary-selection t]
    ))

(defun ewb-menu-filter (menu-items)
  "\"edit-menu-filter\"ɲáʲξactiveˤʤ롣

  Make/Edit index  //in??ȥꥬξ˥ݥ󥿤뤫ΰꤷƤ
  Jumpt to entry   //in??ȥꥬξ˥ݥ󥿤
  Remove index     //in??ȥꥬξ˥ݥ󥿤
  View Figure      ɽȥꥬξ˥ݥ󥿤


For use as an incremental menu construction filter.
This function changes the sensitivity of these Edit menu items:

  Cut    sensitive only when emacs owns the primary X Selection.
  Copy   sensitive only when emacs owns the primary X Selection.
  Clear  sensitive only when emacs owns the primary X Selection.
  Paste  sensitive only when there is an owner for the X Clipboard Selection.
  Undo   sensitive only when there is undo information.
         While in the midst of an undo, this is changed to \"Undo More\"."
  (let* (item
	name
	(result menu-items)		; save pointer to head of list
	(x-dev (eq 'x (device-type (selected-device))))
	(emacs-owns-selection-p (and x-dev (x-selection-owner-p)))
	(clipboard-exists-p (and x-dev (x-selection-exists-p 'CLIPBOARD)))
;;;       undo-available undoing-more
;;;       (undo-info-available (not (null (and (not (eq t buffer-undo-list))
;;;                                 (if (eq last-command 'undo)
;;;                                     (setq undoing-more
;;;                                           (and (boundp 'pending-undo-list)
;;;                                          pending-undo-list)
;;;                                   buffer-undo-list))))))
	undo-name undo-state
	)
    ;; As with file-menu-filter, menu-items are destructively modified.
    ;; This is OK.
    (while (setq item (pop menu-items))
      (if (vectorp item)
	  (progn
	    (setq name (aref item 0))
	    (and (string= name "Make/Edit index")
		 (aset item 2 (or emacs-owns-selection-p
				  (ewb-on-in-triger-p))))
	    (and (string= name "View Figure")
		 (aset item 2 (ewb-current-figure-triger)))
	    (and (member name '("Jumpt to entry" "Remove index"))
		 (aset item 2 (ewb-on-in-triger-p)))
	    (and (member name '("Cut" "Copy" "Clear"))
		 (aset item 2 emacs-owns-selection-p))
	    (and (string= name "Paste")
		 (aset item 2 clipboard-exists-p))
	    (and (member name '("Undo" "Undo More"))
		 (progn
		   ;; we could also do this with the third field of the item.
		   (if (eq last-command 'undo)
		       (setq undo-name "Undo More"
			     undo-state (not (null (and (boundp 'pending-undo-list)
							pending-undo-list))))
		     (setq undo-name "Undo"
			   undo-state (and (not (eq buffer-undo-list t))
					   (not (null
						 (or buffer-undo-list
						     (and (boundp 'pending-undo-list)
							  pending-undo-list)))))))
		   (if buffer-read-only (setq undo-state nil))
		   (aset item 0 undo-name)
		   (aset item 2 undo-state)
		   ))
      )))
    result))


(defun ewb-shelf-menu-filter (menu)
  ".shelf2ե򤹤ե륿"
  (mapcar (lambda (file-name)
	    `[ ,file-name
	       (find-file ,file-name) t])
	  (ewb-shelf-file-list)))


(defun ewb-shelf-file-list ()
  ".shelf2ˤե̾ΥꥹȤ֤"
  (let ((list)
	(obuf (current-buffer))
	(sbuf (find-file-noselect ewb-shelf-file-name)))
    (progn
      (set-buffer sbuf)
      (goto-char (point-min))
      (while (re-search-forward "^[^ ]+ +\\([^ ]+\\) " nil t nil)
	(setq list (cons (match-string 1)
			 list)))
      (kill-buffer sbuf)
      (set-buffer obuf)
      (reverse list))))


(require `ewb-forall)
(setq auto-mode-alist
      (cons
       '("ForAll\\.index$" . ewb-forall-index-mode)
       auto-mode-alist))

(run-hooks 'ewb-mode-load-hook)

(provide 'ewb-mode)

;;; Local Variables:
;;; mode: outline-minor
;;; End:

;;; ewb-mode.el ends here
